from . import *
from sys import argv
import re, time



def run():
	if("--version" in argv):
		print(f"SitemapGen {VERSION} - By Nalin Angrish.")
		exit(0)

	if("--help" in argv):
		displayHelpMessage(VERSION)
		exit(0)

		

	try:
		site = prepare(argv[argv.index("--url")+1])
		disguise = site
		if("--disguise" in argv):
			disguise = prepare(argv[argv.index("--disguise")+1])
		output = argv[argv.index("--out")+1]
	except ValueError as e:
		errorkey = re.findall("'.*'", str(e))[0]
		print("Cannot find a required parameter: " + errorkey + ". Use \"sitemapgen --help\" for more information about how to use the command")
		exit(1)

	print("Generating sitemap for URL: \""+site+"\"")
	if(disguise!=site):
		print("Disguising all URLs to the domain: \""+disguise+"\"")
	print("The output File is present/would be created at: \""+output+"\"")

	time.sleep(2)

	generator = Generator(site, output, disguise)

	print("Discovering URLs.....")
	urls = generator.discover()
	print(f"Discovered {str(len(urls))} URLs.....")
	time.sleep(2)

	print("Generating sitemap.....")
	sitemap = generator.genSitemap()
	time.sleep(2)
	print("Sitemap Generated.....")

	print("Writing Sitemap to output file.....")
	generator.write()
	time.sleep(2)
	print("Sitemap successfully written.....")


from requests import get
from bs4 import BeautifulSoup
from .helper import *


VERSION = "v0.9.1"
AUTHOR = "Nalin Angrish"
SOURCE = "https://github.com/Nalin-2005/SitemapGen"
AUTHOR_WEBSITE = "https://www.nalinangrish.me"




class Generator():
	def __init__(self, site, output, disguise=None) -> None:
		self.site = site
		if(disguise!=None):
			self.disguise = disguise
		else:
			self.disguise = site
		self.output = output
		
	def genSitemap(self) -> str:
		sitemap = header
		for url in self.urls:
			sitemap += siteFormat.format(str(url), str(timestamp))
		sitemap += footer
		self.sitemap = sitemap
		return sitemap		


	def getLinks(self, path) -> list:
		url = self.site + path
		page = get(url).text
		soup = BeautifulSoup(page, features="html.parser")
		linktags = soup.findAll("a")
		links = []
		for linktag in linktags:
			links.append(linktag["href"])
		return filter(links)

	def discover(self) -> list:
		urls = []
		links = self.getLinks("/")
		passed = []
		left = True
		while(left):
			left = False
			xlinks = []
			for link in links:
				if link not in passed:
					urls.append(self.disguise+link)
					xlinks.extend(self.getLinks(link))
					passed.append(link)
					left = True
			links = xlinks
		
		self.urls = urls
		return urls

	def write(self):
		with open(self.output, "w+") as file:
			file.write(self.sitemap)


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

with open("requirements.txt", "r") as fh:
	requirements = fh.read().splitlines()


setuptools.setup(
    name="sitemapgen",
    version="0.9.1",
    author="Nalin Angrish",
    author_email="nalin@nalinangrish.me",
    description="A package to generate Sitemaps from a URL. Also provides a CLI for non programmatical use.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Nalin-2005/SitemapGen",
    entry_points='''
        [console_scripts]
        sitemapgen=sitemapgen.cli:run
    ''',
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=requirements,
    python_requires='>=3.0',
)


from datetime import date




siteFormat = """
<url>
<loc>{}</loc>
<lastmod>{}</lastmod>
<priority>1</priority>
</url>
"""
header = '<urlset xmlns="http://www.sitemaps.org/schemas/sitemap/0.9" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://www.sitemaps.org/schemas/sitemap/0.9 http://www.sitemaps.org/schemas/sitemap/0.9/sitemap.xsd">\n<!-- Made using SitemapGen - By Nalin Angrish -->'
footer = "</urlset>"
timestamp = date.today()







def filter(array):
	links = list(dict.fromkeys(array))
	finalLinks = []
	for link in links:
		if((not str(link).startswith("http")) and (not ":" in str(link)) and (not str(link).startswith("#"))):
			if(link.startswith("/")):
				finalLinks.append(link)
			else:
				finalLinks.append("/"+link)
	return finalLinks




def displayHelpMessage(VERSION):
	print(f"""SitemapGen {VERSION} - By Nalin Angrish.
A general utility script for generating site XML sitemaps.

Options:
  --version        | Show the tool version
  --help           | Show this message and exit.
  --url <url>      | Specify a website url to generate a sitemap from. 
  --out <path>     | Specify an output file for the sitemap. 
  --disguise <url> | Specify a disguise URL for use in the sitemap. Useful when you are creating sitemap for a local website before hosting it.
  

When Running the command, you need to specify the '--url' and the '--out' parameters while the '--disguise' parameter is optional. 
Also, running the command with --version or --help will lead to the suppression of other parameters.""")


def prepare(link:str):
	if(link.endswith("/")):
		link = link[:-1]
	if("http" not in link):
		raise Exception(f"{link} is not a valid URL!")
	return link




